
/*
 *   OpenJupes v1.1
 *
 *   Server and Nickname Jupe Service for Ratbox v2+services.
 *   Copyright (C) 2005 Thomas Carlsson, Joost Vunderink and Alan Milford
 *
 *   NOTE: This service depends heavily on the +services patch by Lee
 *   Hardy. It will not compile without it.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */




#include "stdinc.h"
#include "client.h"
#include "irc_string.h"
#include "send.h"
#include "numeric.h"
#include "ircd.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_serv.h"
#include "hash.h"
#include "s_conf.h"
#include "hook.h"


#include <time.h>
#include <string.h>

#include "oj_main.h"





static char conffilename[] = "openjupes/openjupes.conf";



static char OJ_reply[IRCLINELEN * 5];	



void CONF_Load( struct OJ_Conf* conf )
{
	FILE* f;
	static char parseline[7][256];
	static char* parsebuf;
	int lines, pl, pm, p, ignoreline, size;
	int countj, counta, counti, countn;
	int curj, cura, curi, curn;

	if ( !conf ) return;
	conf->ok = 0;

	f = fopen( conffilename, "rb" );
	if ( !f )
	{
		OJ_IRCD_BroadcastLocal( "OpenJupes::LoadConf - FATAL - unable to open configuration file for reading." );
		return;
	}

	fseek( f, 0, SEEK_END );
	size = ftell( f );

	if ( size <= 0 )
	{
		fclose( f );
		OJ_IRCD_BroadcastLocal( "OpenJupes::LoadConf - FATAL - configuration file appears to be empty." );
		return;
	}
	if ( size > 10000000 )
	{
		fclose( f );
		OJ_IRCD_BroadcastLocal( "OpenJupes::LoadConf - FATAL - your configuration file appears to be >10 Mb in size. Can this be correct?" );
		return;
	}

	fseek( f, 0, SEEK_SET );

	parsebuf = (char*)MyMalloc( size );
	if ( !parsebuf )
	{
		fclose( f );
		OJ_IRCD_BroadcastLocal( "OpenJupes::LoadConf - FATAL - out of memory." );
		return;
	}

	lines = 0;
	pl = 0;
	pm = 0;
	ignoreline = 0;

	fread( parsebuf, size, 1, f );

	/* Let's free the old lines */

	/* Let's count the amount of lines needed */
	countj = 0;
	counta = 0;
	counti = 0;
	countn = 0;
	for ( p = 0; p < size; p++ )
	{
		if ( ignoreline )
		{
			if ( parsebuf[p] != '\n' ) continue;
			else
			{
				pm = 0;
				pl = 0;
				ignoreline = 0;
				continue;
			}
		}
		/* Ignore whitespace */
		if ( parsebuf[p] == '\r' )
		{
			continue;
		}
		if ( ( parsebuf[p] == ' ' || parsebuf[p] == '\t' ) )
		{
			continue;
		}
		if ( parsebuf[p] == '#' || parsebuf[p] == ';' )
		{
			ignoreline = 1; 
			continue;
		}
		if ( parsebuf[p] == '\n' )
		{
			pm = 0;
			pl = 0;
			continue;
		}

		if ( parsebuf[p] == ':' )
		{
			if ( pm == 1 )
			{
				/* <char>: pattern found */
				switch ( parseline[0][0] )
				{
					case 'j':
					case 'J':
					{
						countj++;
						break;
					}
					case 'a':
					case 'A':
					{
						counta++;
						break;
					}
					case 'i':
					case 'I':
					{
						counti++;
						break;
					}
					case 'n':
					case 'N':
					{
						countn++;
						break;
					}
				}
			}
			else
			{
/*				sprintf( OJ_reply, "OpenJupes::LoadConf - DEBUG: %s", parseline[0] );
				OJ_IRCD_BroadcastLocal( OJ_reply );*/
			}
			/* Just count, and move on for now */
			parseline[pl][pm] = 0;
			pm = 0;
			pl = 0;
			ignoreline = 1;
		}
		else
		{
			parseline[pl][pm] = parsebuf[p];
			pm++;

			if ( pm > 40 )
			{
				pm = 0;
				pl = 0;	
				parseline[pl][0] = 0;
				ignoreline = 1;
			}
		}
	}

	sprintf( OJ_reply, "OpenJupes::LoadConf - J-lines: %d A-lines: %d I-lines: %d N-lines: %d", countj, counta, counti, countn );
	OJ_IRCD_BroadcastLocal( OJ_reply );

	if ( countj > 1 )
	{
		/* This is wrong, so let the user know about it .. */
		OJ_IRCD_BroadcastLocal( "OpenJupes::LoadConf - It seems like you've defined more than one J-line. Only the first one will be processed." );
	}
	if ( countj < 1 )
	{
		/* This is also not really ideal, so let the user know about it .. */
		OJ_IRCD_BroadcastLocal( "OpenJupes::LoadConf - You've not defined a J-line, so I'll resort to the default nick/user/host for the service." );

		strcpy( conf->client.nick, JUPE_MYNICK );
		strcpy( conf->client.user, JUPE_MYUSER );
		strcpy( conf->client.host, JUPE_MYHOST );
		strcpy( conf->client.gecos, JUPE_MYGECOS );

		/* Actually, halt proceedings for now - risk of continuing too big */
		return;
	}

	/* Reserve needed memory */
	conf->admin			= (struct OJ_Conf_Admins*)MyMalloc( sizeof( struct OJ_Conf_Admins ) * counta );
	conf->admincount	= 0;
	conf->ignore		= (struct OJ_Conf_Ignores*)MyMalloc( sizeof( struct OJ_Conf_Ignores ) * counti );
	conf->ignorecount	= 0;
	conf->nick			= (struct OJ_Conf_Nicks*)MyMalloc( sizeof( struct OJ_Conf_Nicks ) * countn );
	conf->nickcount		= 0;

	/* Let's reprocess all conf file entries */
	curj = 0;
	cura = 0;
	curi = 0;
	curn = 0;

	lines = 0;
	pl = 0;
	pm = 0;
	ignoreline = 0;
	for ( p = 0; p < size; p++ )
	{
		if ( ignoreline )
		{
			if ( parsebuf[p] != '\n' ) continue;
			else
			{
				ignoreline = 0;
				continue;
			}
		}
		/* Ignore whitespace */
		if ( parsebuf[p] == '\r' )
		{
			continue;
		}
		if ( ( parsebuf[p] == ' ' || parsebuf[p] == '\t' ) && pl < 2 )
		{
			continue;
		}

		if ( parsebuf[p] == '\n' )
		{
			/* Linefeed encountered .. Let's see what we got */
			if ( pl > 0 )
			{
				/* Force EOL to the currently parsed parameter bucket */
				parseline[pl][pm] = 0;

				if ( parseline[0][0] && !parseline[0][1] )
				{
					switch ( parseline[0][0] )
					{
						default:
						{
							sprintf( OJ_reply, "OpenJupes::LoadConf - Unknown configuration token %s (skipping)", parseline[0] );
							OJ_IRCD_BroadcastLocal( OJ_reply );
							break;
						}
						case 'j':
						case 'J':
						{
							if ( pl != 4 )
							{
								sprintf( OJ_reply, "OpenJupes::LoadConf - Wrong number of parameters for J-line (4 expected, got %d)", pl );
								OJ_IRCD_BroadcastLocal( OJ_reply );
							}
							else
							{
								if ( curj < 1 )
								{
/*									sprintf( OJ_reply, "DEBUG: Read J-line: %s,%s,%s,%s", 
										parseline[1], parseline[2], parseline[3], parseline[4] );
									OJ_IRCD_BroadcastLocal( OJ_reply );*/

									strcpy( conf->client.nick, parseline[1] );
									strcpy( conf->client.user, parseline[2] );
									strcpy( conf->client.host, parseline[3] );
									strcpy( conf->client.gecos, parseline[4] );

									lines++;
									curj++;
								}
							}
							break;
						}
						case 'a':
						case 'A':
						{
							if ( pl != 3 )
							{
								sprintf( OJ_reply, "OpenJupes::LoadConf - Wrong number of parameters for A-line (3 expected, got %d)", pl );
								OJ_IRCD_BroadcastLocal( OJ_reply );
							}
							else
							{
								if ( cura < counta )
								{
/*									sprintf( OJ_reply, "DEBUG: Read A-line: %s,%s,%s", 
										parseline[1], parseline[2], parseline[3] );
									OJ_IRCD_BroadcastLocal( OJ_reply );*/

									strcpy( conf->admin[cura].user, parseline[1] );
									strcpy( conf->admin[cura].host, parseline[2] );
									strcpy( conf->admin[cura].server, parseline[3] );

									lines++;
									cura++;
								}
								else
								{
									/* Should never happen */
									OJ_IRCD_BroadcastLocal( "DEBUG: FATAL logic error - cura overflow!" );
								}
							}
							break;
						}
						case 'i':
						case 'I':
						{
							if ( pl != 1 )
							{
								sprintf( OJ_reply, "OpenJupes::LoadConf - Wrong number of parameters for I-line (1 expected, got %d)", pl );
								OJ_IRCD_BroadcastLocal( OJ_reply );
							}
							else
							{
								if ( curi < counti )
								{
/*									sprintf( OJ_reply, "DEBUG: Read I-line: %s", 
										parseline[1] );
									OJ_IRCD_BroadcastLocal( OJ_reply );*/

									strcpy( conf->ignore[curi].server, parseline[1] );

									lines++;
									curi++;
								}
								else
								{
									/* Should never happen */
									OJ_IRCD_BroadcastLocal( "DEBUG: FATAL logic error - curi overflow!" );
								}
							}
							break;
						}
						case 'n':
						case 'N':
						{
							if ( pl != 2 )
							{
								sprintf( OJ_reply, "OpenJupes::LoadConf - Wrong number of parameters for N-line (2 expected, got %d)", pl );
								OJ_IRCD_BroadcastLocal( OJ_reply );
							}
							else
							{
								if ( curn < countn )
								{
/*									sprintf( OJ_reply, "DEBUG: Read N-line: %s,%s", 
										parseline[1], parseline[2] );
									OJ_IRCD_BroadcastLocal( OJ_reply );*/
									
									strcpy( conf->nick[curn].nick, parseline[1] );
									strcpy( conf->nick[curn].gecos, parseline[2] );

									lines++;
									curn++;
								}
								else
								{
									/* Should never happen */
									OJ_IRCD_BroadcastLocal( "DEBUG: FATAL logic error - curn overflow!" );
								}
							}
							break;
						}
					}
				}
				else
				{
					sprintf( OJ_reply, "OpenJupes::LoadConf - Unknown configuration token %s (skipping)", parseline[0] );
					OJ_IRCD_BroadcastLocal( OJ_reply );
				}
				pl = 0;
				pm = 0;
			}
		}
		else
		if ( !pl ) 			/* The first section */
		{
			if ( parsebuf[p] == '#' || parsebuf[p] == ';' )
			{
				/* Comment char encountered.. let's just ignore the rest of the line */
				pm = 0;
				pl = 0;
				ignoreline = 1; 
				continue;
			}
			if ( parsebuf[p] == ':' )
			{
				parseline[pl][pm] = 0;
				pm = 0;
				pl++;	/* Okay, moving on to next section */
			}
			else
			{
				parseline[pl][pm] = parsebuf[p];
				pm++;

				if ( pm > 240 )
				{
					/* Uberflow */
					pm = 0;
					pl = 0;	/* Redundant, but anyway.. */
					parseline[pl][0] = 0;
					ignoreline = 1;
				}
			}
		}
		else
		{
			if ( parsebuf[p] == ',' )
			{
				parseline[pl][pm] = 0;
				pm = 0;
				pl++;	/* Okay, moving on to next section */

				if ( pl > 6 )
				{
					/* Uberflow, ignore */
					pl = 0;
					pm = 0;
					parseline[0][0] = 0;
					ignoreline = 1;
				}
			}
			else
			{
				parseline[pl][pm] = parsebuf[p];
				pm++;

				if ( pm > 240 )
				{
					/* Uberflow, ignore */
					pl = 0;
					pm = 0;
					parseline[0][0] = 0;
					ignoreline = 1;
				}
			}
		}
	}


	conf->admincount	= cura;
	conf->ignorecount	= curi;
	conf->nickcount		= curn;

	sprintf( OJ_reply, "OpenJupes::LoadConf - Successfully read %d lines from configuration file.", lines );
	OJ_IRCD_BroadcastLocal( OJ_reply );


	MyFree( parsebuf );


	conf->ok = 1;

}




void CONF_Free( struct OJ_Conf* conf )
{
	if ( conf->admin ) MyFree( conf->admin );
	if ( conf->ignore ) MyFree( conf->ignore );
	if ( conf->nick ) MyFree( conf->nick );
	
	conf->admin = NULL;
	conf->ignore = NULL;
	conf->nick = NULL;
	
	conf->admincount = 0;
	conf->ignorecount = 0;
	conf->nickcount = 0;
}



/* Lame helper functions. Didn't know where to put these, so I put them here. */




int CONF_IsIgnore( struct OJ_Conf* conf, char* server )
{
	int i;

	if ( !conf->ok ) return 0;

	for ( i = 0; i < conf->ignorecount; i++ )
	{
		if ( !strcasecmp( conf->ignore[i].server, server ) )
		{
			return 1;
		}
	}

	return 0;
}

int CONF_IsAdmin( struct OJ_Conf* conf, char* user, char* host, char* server )
{
	int i;

	if ( !conf->ok ) return 0;

	for ( i = 0; i < conf->admincount; i++ )
	{
		if ( !strcasecmp( conf->admin[i].server, server ) &&
			 !strcasecmp( conf->admin[i].user, user ) && 
			 !strcasecmp( conf->admin[i].host, host ) )
		{
			return 1;
		}
	}

	return 0;
}

int CONF_Rehash( struct OJ_Conf* conf )
{
	struct OJ_Conf* tconf;

	/* Okay, let's see if there's a valid conf for us to read. We will refuse to
	 * overwrite the existing conf with junk. 
	 */

	tconf = (struct OJ_Conf*)MyMalloc( sizeof( struct OJ_Conf ) );
	CONF_Load( tconf );
	if ( !tconf->ok )
	{
		MyFree( tconf );
		OJ_IRCD_BroadcastLocal( "CONF::Rehash - Fatal errors encountered when re-reading conf file, keeping old configuration." );
		return 0;
	}

	/* New conf loaded ok, so we'll take it */
	OJ_DeleteNickJupes( conf );
	CONF_Free( conf );

	memcpy( conf, tconf, sizeof( struct OJ_Conf ) );

	OJ_CreateNickJupes( conf );	

	MyFree( tconf );

	return 1;
}

